package com.staffing.framework.web.service;

import javax.annotation.Resource;

import com.staffing.common.core.domain.WxLogin;
import com.staffing.common.core.domain.WxPhone;
import com.staffing.common.core.domain.WxToken;
import com.staffing.common.enums.UserStatus;
import com.staffing.common.utils.WxUtil;
import com.staffing.custom.domain.StEmployee;
import com.staffing.custom.service.IStEmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;
import com.staffing.common.constant.CacheConstants;
import com.staffing.common.constant.Constants;
import com.staffing.common.constant.UserConstants;
import com.staffing.common.core.domain.entity.SysUser;
import com.staffing.common.core.domain.model.LoginUser;
import com.staffing.common.core.redis.RedisCache;
import com.staffing.common.exception.ServiceException;
import com.staffing.common.exception.user.BlackListException;
import com.staffing.common.exception.user.CaptchaException;
import com.staffing.common.exception.user.CaptchaExpireException;
import com.staffing.common.exception.user.UserNotExistsException;
import com.staffing.common.exception.user.UserPasswordNotMatchException;
import com.staffing.common.utils.DateUtils;
import com.staffing.common.utils.MessageUtils;
import com.staffing.common.utils.StringUtils;
import com.staffing.common.utils.ip.IpUtils;
import com.staffing.framework.manager.AsyncManager;
import com.staffing.framework.manager.factory.AsyncFactory;
import com.staffing.framework.security.context.AuthenticationContextHolder;
import com.staffing.system.service.ISysConfigService;
import com.staffing.system.service.ISysUserService;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;

/**
 * 登录校验方法
 * 
 * @author staffing
 */
@Component
public class SysLoginService
{
    @Autowired
    private TokenService tokenService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;
    
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;
    @Autowired
    private IStEmployeeService stEmployeeService;
    @Autowired
    private SysPermissionService permissionService;
    @Autowired
    private WxUtil wxUtil;

    /**
     * 登录验证
     * 
     * @param username 用户名
     * @param password 密码
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid)
    {
        // 验证码校验
        validateCaptcha(username, code, uuid);
        // 登录前置校验
        loginPreCheck(username, password);
        // 用户验证
        Authentication authentication = null;
        try
        {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
            AuthenticationContextHolder.setContext(authenticationToken);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordLoginInfo(loginUser.getUserId());
        // 生成token
        return tokenService.createToken(loginUser);
    }
    /**
     * 登录验证
     * @return 结果
     */
    public String wxLogin(String loginCode, String phoneCode, int type)
    {
        if (type == SysUser.ROLETYPE.EMPLOYEE.ordinal()){
            return wxEmployeeLogin(loginCode, phoneCode);
        }else{
            return wxMerchantLogin(loginCode, phoneCode);
        }
    }

    /**
     * 职工端 登录
     * @param loginCode
     * @param phoneCode
     * @return
     */
    public String wxEmployeeLogin(String loginCode, String phoneCode)
    {

        // 登录前置校验
//        loginPreCheck(username, password);
        // 用户验证
        LoginUser loginUser = null;
        try
        {
            WxLogin login = wxUtil.login(loginCode);
            WxPhone wxPhone = wxUtil.getPhoneNumber(phoneCode, login.getOpenid());
            String phoneNumber = wxPhone.getPhoneNumber();

            StEmployee stEmployee = stEmployeeService.selectStEmployeeByPhone(phoneNumber);
            if (stEmployee == null){
                stEmployee = new StEmployee();
                stEmployee.setPhone(phoneNumber);
                stEmployee.setOpenId(login.getOpenid());
                stEmployee.setRealName("微信用户");
                stEmployee.setCreateTime(new Date());
                stEmployee.setUpdateTime(new Date());
                stEmployeeService.insertStEmployee(stEmployee);
            }
            if (stEmployee.getOpenId() == null){
                stEmployee.setOpenId(login.getOpenid());
                stEmployeeService.updateStEmployee(stEmployee);
            }

            SysUser user = new SysUser();
            user.setUserId(stEmployee.getId());
            user.setPhonenumber(stEmployee.getPhone());
            user.setIdCard(stEmployee.getIdCard());
            user.setNickName(stEmployee.getRealName());
            user.setOpenId(stEmployee.getOpenId());
            user.setRoleType(SysUser.ROLETYPE.EMPLOYEE);
            loginUser =  new LoginUser(stEmployee.getId(), 0l, user, new HashSet<>());
        }
        catch (Exception e)
        {
            throw e;
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        // 生成token
        return tokenService.createToken(loginUser);
    }
    /**
     * 登录验证
     * @return 结果
     */
    public String wxMerchantLogin(String loginCode, String phoneCode)
    {

        // 登录前置校验
//        loginPreCheck(username, password);
        // 用户验证
        LoginUser loginUser = null;
        try
        {
            WxLogin login = wxUtil.login(loginCode);
            WxPhone wxPhone = wxUtil.getPhoneNumber(phoneCode, login.getOpenid());
            String phoneNumber = wxPhone.getPhoneNumber();

            SysUser user = userService.selectUserByPhone(phoneNumber);
            if (StringUtils.isNull(user))
            {
                throw new ServiceException(MessageUtils.message("user.not.exists"));
            }
            else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
            {
                throw new ServiceException(MessageUtils.message("user.password.delete"));
            }
            else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
            {
                throw new ServiceException(MessageUtils.message("user.blocked"));
            }
            user.setRoleType(SysUser.ROLETYPE.MERCHANT);
            loginUser =  createLoginUser(user);
            recordLoginInfo(loginUser.getUserId());
        }
        catch (Exception e)
        {
            throw e;
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        // 生成token
        return tokenService.createToken(loginUser);
    }
    public LoginUser createLoginUser(SysUser user)
    {
        return new LoginUser(user.getUserId(), user.getDeptId(), user, permissionService.getMenuPermission(user));
    }

    /**
     * 校验验证码
     * 
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        if (captchaEnabled)
        {
            String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
            String captcha = redisCache.getCacheObject(verifyKey);
            if (captcha == null)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
                throw new CaptchaExpireException();
            }
            redisCache.deleteObject(verifyKey);
            if (!code.equalsIgnoreCase(captcha))
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
                throw new CaptchaException();
            }
        }
    }

    /**
     * 登录前置校验
     * @param username 用户名
     * @param password 用户密码
     */
    public void loginPreCheck(String username, String password)
    {
        // 用户名或密码为空 错误
        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("not.null")));
            throw new UserNotExistsException();
        }
        // 密码如果不在指定范围内 错误
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // 用户名不在指定范围内 错误
        if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // IP黑名单校验
        String blackStr = configService.selectConfigByKey("sys.login.blackIPList");
        if (IpUtils.isMatchedIp(blackStr, IpUtils.getIpAddr()))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("login.blocked")));
            throw new BlackListException();
        }
    }

    /**
     * 记录登录信息
     *
     * @param userId 用户ID
     */
    public void recordLoginInfo(Long userId)
    {
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        sysUser.setLoginIp(IpUtils.getIpAddr());
        sysUser.setLoginDate(DateUtils.getNowDate());
        userService.updateUserProfile(sysUser);
    }
}
